@@ -6,6 +6,7 @@ from dataclasses import asdict
 import tornado
 from jupyter_server.base.handlers import APIHandler
 from jupyter_server.extension.handler import ExtensionHandlerMixin
+from jupyter_server.utils import ensure_async
 
 from jupyter_scheduler.environments import EnvironmentRetrievalError
 from jupyter_scheduler.models import (
@@ -13,11 +14,14 @@ from jupyter_scheduler.models import (
     DEFAULT_SORT,
     CountJobsQuery,
     CreateJob,
+    CreateJobDefinition,
+    ListJobDefinitionsQuery,
     ListJobsQuery,
     SortDirection,
     SortField,
     Status,
     UpdateJob,
+    UpdateJobDefinition,
 )
 
 
@@ -49,33 +53,69 @@ class JobHandlersMixin:
         return self.execution_config.execution_manager_class
 
 
+def compute_sort_model(query_argument):
+    sort_by = []
+    PATTERN = re.compile("^(asc|desc)?\\(?([^\\)]+)\\)?", re.IGNORECASE)
+    for query in query_argument:
+        m = re.match(PATTERN, query)
+        sort_dir, name = m.groups()
+        sort_by.append(
+            SortField(
+                name=name,
+                direction=SortDirection(sort_dir.lower()) if sort_dir else SortDirection.asc,
+            )
+        )
+
+    return sort_by
+
+
 class JobDefinitionHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
     @tornado.web.authenticated
-    def get(self, job_definition_id=None):
-        raise tornado.web.HTTPError(500, f"Not implemented yet")
-
-    @tornado.web.authenticated
-    def post(self):
-        raise tornado.web.HTTPError(500, f"Not implemented yet")
-
-
-class CreateJobWithDefinitionHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
-    @tornado.web.authenticated
-    async def post(self, job_definition_id: str):
-        job_definition = self.scheduler.get_job_definition(job_definition_id)
-        if job_definition is None:
-            raise tornado.web.HTTPError(
-                404, f"Job definition with id: {job_definition_id} not found"
-            )
-
-        payload = self.get_json_body()
-        if inspect.isawaitable(self.scheduler.create_job):
-            job_id = await self.scheduler.create_job(
-                CreateJob(**job_definition.dict().merge(payload))
+    async def get(self, job_definition_id=None):
+        if job_definition_id:
+            job_definition = await ensure_async(
+                self.scheduler.get_job_definition(job_definition_id)
             )
+            self.finish(job_definition.json())
         else:
-            job_id = self.scheduler.create_job(CreateJob(**job_definition.dict().merge(payload)))
-        self.finish(json.dumps(dict(job_id=job_id)))
+            create_time = self.get_query_argument("create_time", None)
+            sort_by = compute_sort_model(self.get_query_arguments("sort_by"))
+            list_query = ListJobDefinitionsQuery(
+                job_definition_id=self.get_query_argument("job_definition_id", None),
+                name=self.get_query_argument("name", None),
+                tags=self.get_query_arguments("tags", None),
+                create_time=int(create_time) if create_time else None,
+                sort_by=sort_by if sort_by else [DEFAULT_SORT],
+                max_items=self.get_query_argument("max_items", DEFAULT_MAX_ITEMS),
+                next_token=self.get_query_argument("next_token", None),
+            )
+            list_response = await ensure_async(self.scheduler.list_job_definitions(list_query))
+            self.finish(list_response.json(exclude_none=True))
+
+    @tornado.web.authenticated
+    async def post(self):
+        payload = self.get_json_body()
+        job_definition_id = await ensure_async(
+            self.scheduler.create_job_definition(CreateJobDefinition(**payload))
+        )
+        self.finish(json.dumps(dict(job_definition_id=job_definition_id)))
+
+    @tornado.web.authenticated
+    async def patch(self, job_definition_id):
+        payload = self.get_json_body()
+        await ensure_async(
+            self.scheduler.update_job_definition(
+                UpdateJobDefinition(**payload, job_definition_id=job_definition_id)
+            )
+        )
+        self.set_status(204)
+        self.finish()
+
+    @tornado.web.authenticated
+    async def delete(self, job_definition_id):
+        await ensure_async(self.scheduler.delete_job_definition(job_definition_id))
+        self.set_status(204)
+        self.finish()
 
 
 def compute_sort_model(query_argument):
@@ -98,7 +138,7 @@ class JobHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
     @tornado.web.authenticated
     async def get(self, job_id=None):
         if job_id:
-            job = self.scheduler.get_job(job_id)
+            job = await ensure_async(self.scheduler.get_job(job_id))
             self.finish(job.json())
         else:
             status = self.get_query_argument("status", None)
@@ -114,19 +154,14 @@ class JobHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
                 max_items=self.get_query_argument("max_items", DEFAULT_MAX_ITEMS),
                 next_token=self.get_query_argument("next_token", None),
             )
-            if inspect.isawaitable(self.scheduler.list_jobs):
-                list_jobs_response = await self.scheduler.list_jobs(list_jobs_query)
-            else:
-                list_jobs_response = self.scheduler.list_jobs(list_jobs_query)
+            list_jobs_response = await ensure_async(self.scheduler.list_jobs(list_jobs_query))
+
             self.finish(list_jobs_response.json(exclude_none=True))
 
     @tornado.web.authenticated
     async def post(self):
         payload = self.get_json_body()
-        if inspect.isawaitable(self.scheduler.create_job):
-            job_id = await self.scheduler.create_job(CreateJob(**payload))
-        else:
-            job_id = self.scheduler.create_job(CreateJob(**payload))
+        job_id = await ensure_async(self.scheduler.create_job(CreateJob(**payload)))
 
         self.finish(json.dumps(dict(job_id=job_id)))
 
@@ -139,24 +174,19 @@ class JobHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
 
         status = Status(payload.get("status"))
         if status == Status.STOPPED:
-            if inspect.isawaitable(self.scheduler.stop_job):
-                await self.scheduler.stop_job(job_id)
-            else:
-                self.scheduler.stop_job(job_id)
+            await ensure_async(self.scheduler.stop_job(job_id))
         else:
-            if inspect.isawaitable(self.scheduler.update_job):
-                await self.scheduler.update_job(UpdateJob(job_id=job_id, status=str(status)))
-            else:
+            await ensure_async(
                 self.scheduler.update_job(UpdateJob(job_id=job_id, status=str(status)))
+            )
+
         self.set_status(204)
         self.finish()
 
     @tornado.web.authenticated
     async def delete(self, job_id):
-        if inspect.isawaitable(self.scheduler.delete_job):
-            await self.scheduler.delete_job(job_id)
-        else:
-            self.scheduler.delete_job(job_id)
+        await ensure_async(self.scheduler.delete_job(job_id))
+
         self.set_status(204)
         self.finish()
 
@@ -165,12 +195,8 @@ class BatchJobHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
     @tornado.web.authenticated
     async def delete(self):
         job_ids = self.get_query_arguments("job_id")
-        if inspect.isawaitable(self.scheduler.delete_job):
-            for job_id in job_ids:
-                await self.scheduler.delete_job(job_id)
-        else:
-            for job_id in job_ids:
-                self.scheduler.delete_job(job_id)
+        for job_id in job_ids:
+            await ensure_async(self.scheduler.delete_job(job_id))
 
         self.set_status(204)
         self.finish()
@@ -183,10 +209,8 @@ class JobsCountHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
         count_jobs_query = CountJobsQuery(
             status=Status(status.upper()) if status else Status.IN_PROGRESS
         )
-        if inspect.isawaitable(self.scheduler.count_jobs):
-            count = await self.scheduler.count_jobs(count_jobs_query)
-        else:
-            count = self.scheduler.count_jobs(count_jobs_query)
+        count = await ensure_async(self.scheduler.count_jobs(count_jobs_query))
+
         self.finish(json.dumps(dict(count=count)))
 
 
@@ -204,10 +228,7 @@ class RuntimeEnvironmentsHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHan
         """Returns names of available runtime environments"""
 
         try:
-            if inspect.isawaitable(self.environment_manager.list_environments):
-                environments = await self.environment_manager.list_environments()
-            else:
-                environments = self.environment_manager.list_environments()
+            environments = await ensure_async(self.environment_manager.list_environments())
         except EnvironmentRetrievalError as e:
             raise tornado.web.HTTPError(500, str(e))
 
