@@ -21,7 +21,7 @@ class OutputFormat(BaseModel):
 class RuntimeEnvironment(BaseModel):
     """Defines a runtime context where job
     execution will happen. For example, conda
-    environment
+    environment.
     """
 
     name: str
@@ -47,6 +47,8 @@ class EmailNotifications(BaseModel):
 
 
 class Status(str, Enum):
+    CREATED = "CREATED"
+    QUEUED = "QUEUED"
     IN_PROGRESS = "IN_PROGRESS"
     COMPLETED = "COMPLETED"
     FAILED = "FAILED"
@@ -76,23 +78,16 @@ OUTPUT_FILENAME_TEMPLATE = "{{filename}}-{{timestamp}}"
 class CreateJob(BaseModel):
     """Defines the model for creating a new job"""
 
-    input_uri: str  # input file name or path
-    output_prefix: str  # folder or path where outputs should be stored
-    runtime_environment_name: str  # runtime environment name, see `RuntimeEnvironment`
-    runtime_environment_parameters: Optional[
-        Dict[str, EnvironmentParameterValues]
-    ]  # overrides to runtime environment attributes
-    output_formats: Optional[List[str]] = None  # output formats to be generated
-    idempotency_token: Optional[str] = None  # token to resolve redundant executions
-    job_definition_id: Optional[str] = None  # for jobs created with job definition
-    parameters: Optional[Dict[str, ParameterValues]] = None  # notebook parameters
-    tags: Optional[Tags] = None  # arbitrary tags useful for filtering
-    name: Optional[str] = None  # job name for display
-    email_notifications: Optional[EmailNotifications] = None  # email notifications
-    timeout_seconds: Optional[int] = 600
-    retry_on_timeout: Optional[bool] = False
-    max_retries: Optional[int] = 0
-    min_retry_interval_millis: Optional[int] = 0
+    input_uri: str
+    output_prefix: str
+    runtime_environment_name: str
+    runtime_environment_parameters: Optional[Dict[str, EnvironmentParameterValues]]
+    output_formats: Optional[List[str]] = None
+    idempotency_token: Optional[str] = None
+    job_definition_id: Optional[str] = None
+    parameters: Optional[Dict[str, ParameterValues]] = None
+    tags: Optional[Tags] = None
+    name: Optional[str] = None
     output_filename_template: Optional[str] = OUTPUT_FILENAME_TEMPLATE
     compute_type: Optional[str] = None
 
@@ -101,9 +96,11 @@ class DescribeJob(CreateJob):
     job_id: str
     output_uri: str
     url: str
+    create_time: int
+    update_time: int
     start_time: Optional[int] = None
     end_time: Optional[int] = None
-    status: Status = Status.STOPPED
+    status: Status = Status.CREATED
     status_message: Optional[str] = None
 
     class Config:
@@ -120,7 +117,7 @@ class SortField(BaseModel):
     direction: SortDirection
 
 
-DEFAULT_SORT = SortField(name="start_time", direction=SortDirection.desc)
+DEFAULT_SORT = SortField(name="create_time", direction=SortDirection.desc)
 
 DEFAULT_MAX_ITEMS = 1000
 
@@ -152,46 +149,67 @@ class UpdateJob(BaseModel):
     status: Optional[Status] = None
     status_message: Optional[str] = None
     name: Optional[str] = None
+    compute_type: Optional[str] = None
 
 
 class DeleteJob(BaseModel):
     job_id: str
 
 
-class CreateJobDefinition(CreateJob):
+class CreateJobDefinition(BaseModel):
+    input_uri: str
+    output_prefix: str
+    runtime_environment_name: str
+    runtime_environment_parameters: Optional[Dict[str, EnvironmentParameterValues]]
+    output_formats: Optional[List[str]] = None
+    parameters: Optional[Dict[str, ParameterValues]] = None
+    tags: Optional[Tags] = None
+    name: Optional[str] = None
+    output_filename_template: Optional[str] = OUTPUT_FILENAME_TEMPLATE
+    compute_type: Optional[str] = None
     schedule: Optional[str] = None
     timezone: Optional[str] = None
 
 
 class DescribeJobDefinition(CreateJobDefinition):
     job_definition_id: str
-    last_modified_time: int
-    job_ids: List[str]
+    create_time: int
+    update_time: int
+    active: bool
+
+    class Config:
+        orm_mode = True
 
 
 class UpdateJobDefinition(BaseModel):
     job_definition_id: str
-    last_modified_time: Optional[int] = None
-    schedule: Optional[str] = None
-    input_uri: Optional[str] = None
-    output_prefix: Optional[str] = None
-    runtime_environment_name: Optional[str] = None
-    idempotency_token: Optional[str] = None
+    input_uri: Optional[str]
+    output_prefix: Optional[str]
+    runtime_environment_name: Optional[str]
+    runtime_environment_parameters: Optional[Dict[str, EnvironmentParameterValues]]
+    output_formats: Optional[List[str]] = None
     parameters: Optional[Dict[str, ParameterValues]] = None
     tags: Optional[Tags] = None
     name: Optional[str] = None
-    email_notifications: Optional[EmailNotifications] = None
-    timeout_seconds: Optional[int] = 600
-    max_retries: Optional[int] = 0
-    min_retry_interval_millis: Optional[int] = 0
-    retry_on_timeout: Optional[bool] = False
     url: Optional[str] = None
     timezone: Optional[str] = None  # Should be a timezone e.g., US/Eastern, Asia/Kolkata
     output_filename_template: Optional[str] = OUTPUT_FILENAME_TEMPLATE
+    active: bool
 
 
 class ListJobDefinitionsQuery(BaseModel):
-    job_definition_id: str
+    name: Optional[str] = None
+    create_time: Optional[int] = None
+    tags: Optional[Tags] = None
+    sort_by: List[SortField] = [DEFAULT_SORT]
+    max_items: Optional[int] = DEFAULT_MAX_ITEMS
+    next_token: Optional[str] = None
+
+
+class ListJobDefinitionsResponse(BaseModel):
+    job_definitions: List[DescribeJobDefinition] = []
+    total_count: int = 0
+    next_token: Optional[str] = None
 
 
 class JobFeature(str, Enum):
