@@ -1,5 +1,12 @@
-import { InputLabel, MenuItem, Select, SelectChangeEvent } from '@mui/material';
-import React, { useState } from 'react';
+import React from 'react';
+
+import {
+  FormControl,
+  InputLabel,
+  MenuItem,
+  Select,
+  SelectChangeEvent
+} from '@mui/material';
 
 import { Scheduler } from '../handler';
 import { useTranslator } from '../hooks';
@@ -9,42 +16,36 @@ export type EnvironmentPickerProps = {
   name: string;
   id: string;
   onChange: (event: SelectChangeEvent<string>) => void;
-  environmentsPromise: Promise<Scheduler.IRuntimeEnvironment[]>;
+  environmentList: Scheduler.IRuntimeEnvironment[];
   initialValue: string;
 };
 
 export function EnvironmentPicker(props: EnvironmentPickerProps): JSX.Element {
-  const [environmentList, setEnvironmentList] = useState(
-    [] as Scheduler.IRuntimeEnvironment[]
-  );
   const trans = useTranslator('jupyterlab');
 
-  React.useEffect(() => {
-    props.environmentsPromise.then(envList => setEnvironmentList(envList));
-  }, []);
-
-  if (environmentList.length === 0) {
+  if (props.environmentList.length === 0) {
     return <em>{trans.__('Loading …')}</em>;
   }
 
   const labelId = `${props.id}-label`;
 
   return (
-    <>
+    <FormControl>
       <InputLabel id={labelId}>{props.label}</InputLabel>
       <Select
         labelId={labelId}
+        label={props.label}
         name={props.name}
         id={props.id}
         onChange={props.onChange}
         value={props.initialValue}
       >
-        {environmentList.map((env, idx) => (
+        {props.environmentList.map((env, idx) => (
           <MenuItem value={env.label} title={env.description} key={idx}>
             {env.name}
           </MenuItem>
         ))}
       </Select>
-    </>
+    </FormControl>
   );
 }
