@@ -25,21 +25,21 @@ function StopButton(props: {
   job: Scheduler.IDescribeJob;
   clickHandler: () => void;
 }): JSX.Element | null {
-  if (props.job.status !== 'IN_PROGRESS') {
-    return null;
-  }
-
   const trans = useTranslator('jupyterlab');
   const buttonTitle = props.job.name
     ? trans.__('Stop "%1"', props.job.name)
     : trans.__('Stop job');
 
   return (
-    <ToolbarButtonComponent
-      onClick={props.clickHandler}
-      tooltip={buttonTitle}
-      icon={stopIcon}
-    />
+    <div
+      style={props.job.status !== 'IN_PROGRESS' ? { visibility: 'hidden' } : {}}
+    >
+      <ToolbarButtonComponent
+        onClick={props.clickHandler}
+        tooltip={buttonTitle}
+        icon={stopIcon}
+      />
+    </div>
   );
 }
 
@@ -63,6 +63,7 @@ function DeleteButton(props: {
 
 function RefillButton(props: {
   job: Scheduler.IDescribeJob;
+  environmentList: Scheduler.IRuntimeEnvironment[];
   showCreateJob: (newModel: ICreateJobModel) => void;
 }): JSX.Element | null {
   const trans = useTranslator('jupyterlab');
@@ -90,12 +91,14 @@ function RefillButton(props: {
       jobName: props.job.name ?? '',
       outputPath: props.job.output_prefix,
       environment: props.job.runtime_environment_name,
-      parameters: jobParameters
+      parameters: jobParameters,
+      createType: 'Job'
     };
 
     // Convert the list of output formats, if any, into a list for the initial state
     const jobOutputFormats = props.job.output_formats;
     const outputFormats = outputFormatsForEnvironment(
+      props.environmentList,
       props.job.runtime_environment_name
     );
     if (jobOutputFormats && outputFormats) {
@@ -118,14 +121,14 @@ function RefillButton(props: {
 }
 
 function Timestamp(props: { job: Scheduler.IDescribeJob }): JSX.Element | null {
-  const start_date: Date | null = props.job.start_time
-    ? new Date(props.job.start_time)
+  const create_date: Date | null = props.job.create_time
+    ? new Date(props.job.create_time)
     : null;
-  const start_display_date: string | null = start_date
-    ? start_date.toLocaleString()
+  const create_display_date: string | null = create_date
+    ? create_date.toLocaleString()
     : null;
 
-  return <>{start_display_date}</>;
+  return <>{create_display_date}</>;
 }
 
 function OutputFiles(props: {
@@ -161,8 +164,9 @@ function OutputFiles(props: {
   );
 }
 
-export function buildTableRow(
+export function buildJobRow(
   job: Scheduler.IDescribeJob,
+  environmentList: Scheduler.IRuntimeEnvironment[],
   app: JupyterFrontEnd,
   showCreateJob: (newModel: ICreateJobModel) => void,
   deleteRow: (id: Scheduler.IDescribeJob['job_id']) => void,
@@ -201,7 +205,11 @@ export function buildTableRow(
           deleteRow(job.job_id);
         }}
       />
-      <RefillButton job={job} showCreateJob={showCreateJob} />
+      <RefillButton
+        job={job}
+        environmentList={environmentList}
+        showCreateJob={showCreateJob}
+      />
     </Stack>
   ];
 
