@@ -9,14 +9,71 @@ export class SchedulerService {
       options.serverSettings || ServerConnection.makeSettings();
   }
 
-  async getJobDefinitions(
+  /**
+   * Serializes a query object into a URI query string. Assumes the keys and
+   * values of the query object as URI-encodable via `encoderURIComponent()`.
+   */
+  private serializeToQueryString<
+    T extends { sort_by?: Scheduler.ISortField[] }
+  >(jobQuery: T): string {
+    return (
+      '?' +
+      (Object.keys(jobQuery) as (keyof T)[])
+        .map(prop => {
+          if (prop === 'sort_by') {
+            const sortList: T['sort_by'] | undefined = jobQuery.sort_by;
+            if (sortList === undefined) {
+              return null;
+            }
+
+            // Serialize sort_by as a series of parameters in the firm dir(name)
+            // where 'dir' is the direction and 'name' the sort field
+            return sortList
+              .map(
+                sort =>
+                  `sort_by=${encodeURIComponent(
+                    sort.direction
+                  )}(${encodeURIComponent(sort.name)})`
+              )
+              .join('&');
+          }
+
+          const value = jobQuery[prop];
+          return `${encodeURIComponent(prop as any)}=${encodeURIComponent(
+            value as any
+          )}`;
+        })
+        .join('&')
+    );
+  }
+
+  async getJobDefinition(
     definition_id: string
-  ): Promise<Scheduler.IDescribeJobDefinition[]> {
+  ): Promise<Scheduler.IDescribeJobDefinition> {
     let data;
-    let query = '';
-    if (definition_id) {
-      query = `/${definition_id}`;
+
+    try {
+      data = await requestAPI(
+        this.serverSettings,
+        `job_definitions/${definition_id}`,
+        {
+          method: 'GET'
+        }
+      );
+    } catch (e: any) {
+      console.error(e);
     }
+    return data as Scheduler.IDescribeJobDefinition;
+  }
+
+  async getJobDefinitions(
+    jobDefintionsQuery: Scheduler.IListJobDefinitionsQuery,
+    definition_id?: string
+  ): Promise<Scheduler.IListJobDefinitionsResponse> {
+    let data;
+    const query = definition_id
+      ? `/${definition_id}`
+      : this.serializeToQueryString(jobDefintionsQuery);
     try {
       data = await requestAPI(this.serverSettings, `job_definitions${query}`, {
         method: 'GET'
@@ -24,11 +81,11 @@ export class SchedulerService {
     } catch (e: any) {
       console.error(e);
     }
-    return data as Scheduler.IDescribeJobDefinition[];
+    return data as Scheduler.IListJobDefinitionsResponse;
   }
 
   async createJobDefinition(
-    definition: Scheduler.IBaseJobDefinition
+    definition: Scheduler.ICreateJobDefinition
   ): Promise<Scheduler.IDescribeJobDefinition> {
     let data;
     try {
@@ -42,6 +99,25 @@ export class SchedulerService {
     return data as Scheduler.IDescribeJobDefinition;
   }
 
+  async deleteJobDefinition(
+    definition_id: string
+  ): Promise<Scheduler.IDescribeJobDefinition> {
+    let data;
+
+    try {
+      data = await requestAPI(
+        this.serverSettings,
+        `job_definitions/${definition_id}`,
+        {
+          method: 'DELETE'
+        }
+      );
+    } catch (e: any) {
+      console.error(e);
+    }
+    return data as Scheduler.IDescribeJobDefinition;
+  }
+
   async getJob(job_id: string): Promise<Scheduler.IDescribeJob> {
     let data;
     let query = '';
@@ -63,41 +139,7 @@ export class SchedulerService {
     job_id?: string
   ): Promise<Scheduler.IListJobsResponse> {
     let data;
-    let query = '';
-
-    if (job_id) {
-      query = `/${job_id}`;
-    } else if (jobQuery) {
-      query =
-        '?' +
-        Object.keys(jobQuery)
-          .map(prop => {
-            if (prop === 'sort_by') {
-              const sortList: Scheduler.ISortField[] | undefined =
-                jobQuery.sort_by;
-              if (sortList === undefined) {
-                return null;
-              }
-
-              // Serialize sort_by as a series of parameters in the firm dir(name)
-              // where 'dir' is the direction and 'name' the sort field
-              return sortList
-                .map(
-                  sort =>
-                    `sort_by=${encodeURIComponent(
-                      sort.direction
-                    )}(${encodeURIComponent(sort.name)})`
-                )
-                .join('&');
-            }
-
-            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
-            // @ts-ignore
-            const value = jobQuery[prop];
-            return `${encodeURIComponent(prop)}=${encodeURIComponent(value)}`;
-          })
-          .join('&');
-    }
+    const query = job_id ? `/${job_id}` : this.serializeToQueryString(jobQuery);
 
     try {
       data = await requestAPI(this.serverSettings, `jobs${query}`, {
@@ -182,6 +224,28 @@ export class SchedulerService {
     }
   }
 
+  async pauseJobDefinition(jobDefId: string): Promise<void> {
+    try {
+      await requestAPI(this.serverSettings, `job_definitions/${jobDefId}`, {
+        method: 'PATCH',
+        body: JSON.stringify({ active: false })
+      });
+    } catch (e) {
+      console.error(e);
+    }
+  }
+
+  async resumeJobDefinition(jobDefId: string): Promise<void> {
+    try {
+      await requestAPI(this.serverSettings, `job_definitions/${jobDefId}`, {
+        method: 'PATCH',
+        body: JSON.stringify({ active: true })
+      });
+    } catch (e) {
+      console.error(e);
+    }
+  }
+
   /**
    * The server settings used to make API requests.
    */
@@ -239,20 +303,31 @@ export namespace SchedulerService {
 }
 
 export namespace Scheduler {
-  export interface IBaseJobDefinition {
-    input_path: string;
-    output_path: string;
+  export interface ICreateJobDefinition {
+    input_uri: string;
+    output_prefix: string;
+    runtime_environment_name: string;
+    runtime_environment_parameters?: { [key: string]: number | string };
+    output_formats?: string[];
+    parameters?: { [key: string]: any };
+    tags?: string[];
     name?: string;
+    output_filename_template?: string;
+    compute_type?: string;
+    schedule?: string;
+    timezone?: string;
   }
 
-  export type IUpdateJobDefinition = IBaseJobDefinition;
-
-  export interface IDescribeJobDefinition extends IBaseJobDefinition {
+  export interface IUpdateJobDefinition extends ICreateJobDefinition {
     job_definition_id: string;
-    last_modified_time: string;
-    job_ids: string[];
     url?: string;
-    name?: string;
+  }
+
+  export interface IDescribeJobDefinition extends ICreateJobDefinition {
+    job_definition_id: string;
+    create_time: number;
+    update_time: number;
+    active: boolean;
   }
 
   export interface IEmailNotifications {
@@ -266,6 +341,7 @@ export namespace Scheduler {
     input_uri: string;
     output_prefix: string;
     runtime_environment_name: string;
+    runtime_environment_parameters?: { [key: string]: number | string };
     idempotency_token?: string;
     job_definition_id?: string;
     parameters?: { [key: string]: any };
@@ -276,12 +352,14 @@ export namespace Scheduler {
     max_retries?: number;
     min_retry_interval_millis?: number;
     retry_on_timeout?: boolean;
-    start_time?: number;
     output_filename_template?: string;
     output_formats?: string[];
+    compute_type?: string;
   }
 
   export type Status =
+    | 'CREATED'
+    | 'QUEUED'
     | 'IN_PROGRESS'
     | 'COMPLETED'
     | 'FAILED'
@@ -294,6 +372,8 @@ export namespace Scheduler {
     url: string;
     status: Status;
     status_message: string;
+    create_time: number;
+    update_time: number;
     start_time?: number;
     end_time?: number;
   }
@@ -328,13 +408,26 @@ export namespace Scheduler {
     total_count: number;
   }
 
+  export interface IListJobDefinitionsQuery {
+    sort_by?: ISortField[];
+    max_items?: number;
+    next_token?: string;
+  }
+
+  export interface IListJobDefinitionsResponse {
+    job_definitions: IDescribeJobDefinition[];
+    next_token?: string;
+    total_count: number;
+  }
+
   export interface IRuntimeEnvironment {
     name: string;
     label: string;
     description: string;
     file_extensions: string[];
     output_formats: IOutputFormat[];
-    metadata: { [key: string]: any };
+    metadata: { [key: string]: string };
+    compute_types: string[] | null;
   }
 
   export interface IOutputFormat {
