@@ -11,10 +11,10 @@ import { calendarMonthIcon } from './components/icons';
 import TranslatorContext from './context';
 import { CreateJob } from './mainviews/create-job';
 import { NotebookJobsList } from './mainviews/list-jobs';
-import { JobDetail } from './mainviews/job-detail';
-import { ICreateJobModel, JobsModel } from './model';
+import { ICreateJobModel, JobsModel, ListJobsView } from './model';
 import { getJupyterLabTheme } from './theme-provider';
-import Scheduler from './tokens';
+import { Scheduler } from './tokens';
+import { DetailView } from './mainviews/detail-view';
 
 export class NotebookJobsPanel extends VDomRenderer<JobsModel> {
   readonly _title?: string;
@@ -24,7 +24,17 @@ export class NotebookJobsPanel extends VDomRenderer<JobsModel> {
   readonly _advancedOptions: React.FunctionComponent<Scheduler.IAdvancedOptionsProps>;
 
   constructor(options: NotebookJobsPanel.IOptions) {
-    super(options.model || new JobsModel({}));
+    super(
+      options.model ||
+        new JobsModel({
+          onModelUpdate: () => {
+            // allow us to invoke private parent method
+            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
+            // @ts-ignore
+            this.renderDOM();
+          }
+        })
+    );
     this.addClass('jp-notebook-jobs-panel');
     const trans = options.translator.load('jupyterlab');
 
@@ -39,21 +49,21 @@ export class NotebookJobsPanel extends VDomRenderer<JobsModel> {
     this.node.setAttribute('aria-label', trans.__('Notebook Jobs'));
   }
 
-  toggleView(): void {
-    if (
-      this.model.jobsView !== 'CreateJob' &&
-      this.model.jobsView !== 'ListJobs'
-    ) {
-      return;
-    }
-
-    this.model.jobsView =
-      this.model.jobsView === 'ListJobs' ? 'CreateJob' : 'ListJobs';
+  showListView(list: ListJobsView): void {
+    this.model.listJobsModel.listJobsView = list;
+    this.model.jobsView = 'ListJobs';
   }
 
   showDetailView(jobId: string): void {
     this.model.jobsView = 'JobDetail';
-    this.model.jobDetailModel.jobId = jobId;
+    this.model.jobDetailModel.detailType = 'Job';
+    this.model.jobDetailModel.id = jobId;
+  }
+
+  showJobDefinitionDetail(jobDefId: string): void {
+    this.model.jobsView = 'JobDetail';
+    this.model.jobDetailModel.detailType = 'JobDefinition';
+    this.model.jobDetailModel.id = jobDefId;
   }
 
   render(): JSX.Element {
@@ -71,7 +81,7 @@ export class NotebookJobsPanel extends VDomRenderer<JobsModel> {
               handleModelChange={newModel =>
                 (this.model.createJobModel = newModel)
               }
-              toggleView={this.toggleView.bind(this)}
+              showListView={this.showListView.bind(this)}
               advancedOptions={this._advancedOptions}
             />
           )}
@@ -83,20 +93,22 @@ export class NotebookJobsPanel extends VDomRenderer<JobsModel> {
                 (this.model.listJobsModel = newModel)
               }
               showCreateJob={showCreateJob}
-              showDetailView={this.showDetailView.bind(this)}
+              showJobDetail={this.showDetailView.bind(this)}
+              showJobDefinitionDetail={this.showJobDefinitionDetail.bind(this)}
             />
           )}
           {this.model.jobsView === 'JobDetail' && (
-            <JobDetail
+            <DetailView
               app={this._app}
               model={this.model.jobDetailModel}
-              handleModelChange={newModel =>
-                (this.model.jobDetailModel = newModel)
-              }
               setCreateJobModel={newModel =>
                 (this.model.createJobModel = newModel)
               }
-              setView={view => (this.model.jobsView = view)}
+              setJobsView={view => (this.model.jobsView = view)}
+              setListJobsView={view => {
+                this.model.listJobsModel.listJobsView = view;
+              }}
+              advancedOptions={this._advancedOptions}
             />
           )}
         </TranslatorContext.Provider>
